@@ -92,12 +92,19 @@ QBluetoothDeviceDiscoveryAgentPrivate::~QBluetoothDeviceDiscoveryAgentPrivate()
     delete adapterBluez5;
 }
 
+//TODO: Qt6 remove the pendingCancel/pendingStart logic as it is cumbersome.
+//      It is a behavior change across all platforms and was initially done
+//      for Bluez. The behavior should be similar to QBluetoothServiceDiscoveryAgent
+//      PendingCancel creates issues whereby the agent is still shutting down
+//      but isActive() below already returns false. This means the isActive() is
+//      out of sync with the finished() and cancel() signal.
+
 bool QBluetoothDeviceDiscoveryAgentPrivate::isActive() const
 {
     if (pendingStart)
         return true;
     if (pendingCancel)
-        return false;
+        return false; //TODO Qt6: remove pending[Cancel|Start] logic (see comment above)
 
     return (adapter || adapterBluez5);
 }
@@ -188,14 +195,15 @@ void QBluetoothDeviceDiscoveryAgentPrivate::start()
     }
 
     QDBusPendingReply<> discoveryReply = adapter->StartDiscovery();
+    discoveryReply.waitForFinished();
     if (discoveryReply.isError()) {
         delete adapter;
         adapter = 0;
         errorString = discoveryReply.error().message();
         lastError = QBluetoothDeviceDiscoveryAgent::InputOutputError;
         Q_Q(QBluetoothDeviceDiscoveryAgent);
-        emit q->error(lastError);
         qCDebug(QT_BT_BLUEZ) << Q_FUNC_INFO << "ERROR: " << errorString;
+        emit q->error(lastError);
         return;
     }
 }
@@ -238,15 +246,22 @@ void QBluetoothDeviceDiscoveryAgentPrivate::startBluez5()
     QDBusPendingReply<ManagedObjectList> reply = managerBluez5->GetManagedObjects();
     reply.waitForFinished();
     if (!reply.isError()) {
-        foreach (const QDBusObjectPath &path, reply.value().keys()) {
-            const InterfaceList ifaceList = reply.value().value(path);
-            foreach (const QString &iface, ifaceList.keys()) {
+        ManagedObjectList managedObjectList = reply.value();
+        for (ManagedObjectList::const_iterator it = managedObjectList.constBegin(); it != managedObjectList.constEnd(); ++it) {
+            const QDBusObjectPath &path = it.key();
+            const InterfaceList &ifaceList = it.value();
+
+            for (InterfaceList::const_iterator jt = ifaceList.constBegin(); jt != ifaceList.constEnd(); ++jt) {
+                const QString &iface = jt.key();
+
                 if (iface == QStringLiteral("org.bluez.Device1")) {
 
                     if (path.path().indexOf(adapterBluez5->path()) != 0)
                         continue; //devices whose path doesn't start with same path we skip
 
                     deviceFoundBluez5(path.path());
+                    if (!isActive()) // Can happen if stop() was called from a slot in user code.
+                      return;
                 }
             }
         }
@@ -407,8 +422,8 @@ void QBluetoothDeviceDiscoveryAgentPrivate::_q_propertyChanged(const QString &na
                 adapter->deleteLater();
                 adapter = 0;
 
-                emit q->canceled();
                 pendingCancel = false;
+                emit q->canceled();
             } else if (pendingStart) {
                 adapter->deleteLater();
                 adapter = 0;
@@ -417,6 +432,10 @@ void QBluetoothDeviceDiscoveryAgentPrivate::_q_propertyChanged(const QString &na
                 pendingCancel = false;
                 start();
             } else {
+                 // happens when agent is created while other agent called StopDiscovery()
+                if (!adapter)
+                    return;
+
                 if (useExtendedDiscovery) {
                     useExtendedDiscovery = false;
                     /* We don't use the Start/StopDiscovery combo here
@@ -426,6 +445,8 @@ void QBluetoothDeviceDiscoveryAgentPrivate::_q_propertyChanged(const QString &na
                     return;
                 }
 
+                QDBusPendingReply<> reply = adapter->StopDiscovery();
+                reply.waitForFinished();
                 adapter->deleteLater();
                 adapter = 0;
                 emit q->finished();
@@ -482,8 +503,8 @@ void QBluetoothDeviceDiscoveryAgentPrivate::_q_discoveryFinished()
     adapterBluez5 = 0;
 
     if (pendingCancel && !pendingStart) {
-        emit q->canceled();
         pendingCancel = false;
+        emit q->canceled();
     } else if (pendingStart) {
         pendingStart = false;
         pendingCancel = false;
@@ -501,7 +522,7 @@ void QBluetoothDeviceDiscoveryAgentPrivate::_q_discoveryInterrupted(const QStrin
         return;
 
     if (path == adapterBluez5->path()) {
-        qCWarning(QT_BT_BLUEZ) << "Device discovery aborted due to unexpected adapter changes";
+        qCWarning(QT_BT_BLUEZ) << "Device discovery aborted due to unexpected adapter changes from another process.";
 
         if (discoveryTimer)
             discoveryTimer->stop();
