@@ -62,10 +62,12 @@ static inline void convertAddress(quint64 from, quint8 (&to)[6])
     to[5] = (from >> 40) & 0xff;
 }
 
-QBluetoothServiceDiscoveryAgentPrivate::QBluetoothServiceDiscoveryAgentPrivate(const QBluetoothAddress &deviceAdapter)
+QBluetoothServiceDiscoveryAgentPrivate::QBluetoothServiceDiscoveryAgentPrivate(
+    QBluetoothServiceDiscoveryAgent *qp, const QBluetoothAddress &deviceAdapter)
 :   error(QBluetoothServiceDiscoveryAgent::NoError), m_deviceAdapterAddress(deviceAdapter), state(Inactive), deviceDiscoveryAgent(0),
     mode(QBluetoothServiceDiscoveryAgent::MinimalDiscovery), singleDevice(false),
-    manager(0), managerBluez5(0), adapter(0), device(0), sdpScannerProcess(0)
+    manager(0), managerBluez5(0), adapter(0), device(0), sdpScannerProcess(0),
+    q_ptr(qp)
 {
     if (isBluez5()) {
         managerBluez5 = new OrgFreedesktopDBusObjectManagerInterface(
@@ -127,13 +129,12 @@ void QBluetoothServiceDiscoveryAgentPrivate::start(const QBluetoothAddress &addr
         }
     }
 
-    QDBusPendingReply<QDBusObjectPath> deviceObjectPath = adapter->CreateDevice(address.toString());
+    QDBusPendingReply<QDBusObjectPath> deviceObjectPath = adapter->FindDevice(address.toString());
 
     QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(deviceObjectPath, q);
     watcher->setProperty("_q_BTaddress", QVariant::fromValue(address));
     QObject::connect(watcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
-                     q, SLOT(_q_createdDevice(QDBusPendingCallWatcher*)));
-
+                     q, SLOT(_q_foundDevice(QDBusPendingCallWatcher*)));
 }
 
 // Bluez 5
@@ -213,15 +214,23 @@ void QBluetoothServiceDiscoveryAgentPrivate::runExternalSdpScan(
 
         sdpScannerProcess = new QProcess(q);
         sdpScannerProcess->setReadChannel(QProcess::StandardOutput);
+        if (QT_BT_BLUEZ().isDebugEnabled())
+            sdpScannerProcess->setProcessChannelMode(QProcess::ForwardedErrorChannel);
         sdpScannerProcess->setProgram(fileInfo.canonicalFilePath());
         q->connect(sdpScannerProcess, SIGNAL(finished(int,QProcess::ExitStatus)),
                    q, SLOT(_q_sdpScannerDone(int,QProcess::ExitStatus)));
-
     }
 
     QStringList arguments;
     arguments << remoteAddress.toString() << localAddress.toString();
 
+    // No filter implies PUBLIC_BROWSE_GROUP based SDP scan
+    if (!uuidFilter.isEmpty()) {
+        arguments << QLatin1String("-u"); // cmd line option for list of uuids
+        foreach (const QBluetoothUuid& uuid, uuidFilter)
+            arguments << uuid.toString();
+    }
+
     sdpScannerProcess->setArguments(arguments);
     sdpScannerProcess->start();
 }
@@ -352,30 +361,65 @@ void QBluetoothServiceDiscoveryAgentPrivate::stop()
     emit q->canceled();
 }
 
-void QBluetoothServiceDiscoveryAgentPrivate::_q_createdDevice(QDBusPendingCallWatcher *watcher)
+void QBluetoothServiceDiscoveryAgentPrivate::_q_foundDevice(QDBusPendingCallWatcher *watcher)
 {
-    if (!adapter)
+    if (!adapter) {
+        watcher->deleteLater();
         return;
+    }
 
     Q_Q(QBluetoothServiceDiscoveryAgent);
 
     const QBluetoothAddress &address = watcher->property("_q_BTaddress").value<QBluetoothAddress>();
 
-    qCDebug(QT_BT_BLUEZ) << Q_FUNC_INFO << "created" << address.toString();
+    qCDebug(QT_BT_BLUEZ) << Q_FUNC_INFO << "found" << address.toString();
 
     QDBusPendingReply<QDBusObjectPath> deviceObjectPath = *watcher;
+    watcher->deleteLater();
     if (deviceObjectPath.isError()) {
-        if (deviceObjectPath.error().name() != QStringLiteral("org.bluez.Error.AlreadyExists")) {
+        if (deviceObjectPath.error().name() != QStringLiteral("org.bluez.Error.DoesNotExist")) {
+            qCDebug(QT_BT_BLUEZ) << "Find device failed Error: " << error << deviceObjectPath.error().name();
             delete adapter;
             adapter = 0;
+            if (singleDevice) {
+                error = QBluetoothServiceDiscoveryAgent::InputOutputError;
+                errorString = QBluetoothServiceDiscoveryAgent::tr("Unable to access device");
+                emit q->error(error);
+            }
             _q_serviceDiscoveryFinished();
-            qCDebug(QT_BT_BLUEZ) << "Create device failed Error: " << error << deviceObjectPath.error().name();
             return;
         }
+
+        deviceObjectPath = adapter->CreateDevice(address.toString());
+        watcher = new QDBusPendingCallWatcher(deviceObjectPath, q);
+        watcher->setProperty("_q_BTaddress",  QVariant::fromValue(address));
+        QObject::connect(watcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
+                         q, SLOT(_q_createdDevice(QDBusPendingCallWatcher*)));
+        return;
+    }
+
+    qCDebug(QT_BT_BLUEZ) << Q_FUNC_INFO << "path" << deviceObjectPath.value().path();
+    discoverServices(deviceObjectPath.value().path());
+}
+
+void QBluetoothServiceDiscoveryAgentPrivate::_q_createdDevice(QDBusPendingCallWatcher *watcher)
+{
+    Q_Q(QBluetoothServiceDiscoveryAgent);
 
-        deviceObjectPath = adapter->FindDevice(address.toString());
-        deviceObjectPath.waitForFinished();
-        if (deviceObjectPath.isError()) {
+    if (!adapter) {
+        watcher->deleteLater();
+        return;
+    }
+
+    const QBluetoothAddress &address = watcher->property("_q_BTaddress").value<QBluetoothAddress>();
+
+    qCDebug(QT_BT_BLUEZ) << Q_FUNC_INFO << "created" << address.toString();
+
+    QDBusPendingReply<QDBusObjectPath> deviceObjectPath = *watcher;
+    watcher->deleteLater();
+    if (deviceObjectPath.isError()) {
+        if (deviceObjectPath.error().name() != QLatin1String("org.bluez.Error.AlreadyExists")) {
+            qCDebug(QT_BT_BLUEZ) << "Create device failed Error: " << error << deviceObjectPath.error().name();
             delete adapter;
             adapter = 0;
             if (singleDevice) {
@@ -384,13 +428,20 @@ void QBluetoothServiceDiscoveryAgentPrivate::_q_createdDevice(QDBusPendingCallWa
                 emit q->error(error);
             }
             _q_serviceDiscoveryFinished();
-            qCDebug(QT_BT_BLUEZ) << "Can't find device after creation Error: " << error << deviceObjectPath.error().name();
             return;
         }
     }
 
+    qCDebug(QT_BT_BLUEZ) << Q_FUNC_INFO << "path" << deviceObjectPath.value().path();
+    discoverServices(deviceObjectPath.value().path());
+}
+
+void QBluetoothServiceDiscoveryAgentPrivate::discoverServices(const QString deviceObjectPath)
+{
+    Q_Q(QBluetoothServiceDiscoveryAgent);
+
     device = new OrgBluezDeviceInterface(QStringLiteral("org.bluez"),
-                                         deviceObjectPath.value().path(),
+                                         deviceObjectPath,
                                          QDBusConnection::systemBus());
     delete adapter;
     adapter = 0;
@@ -479,7 +530,7 @@ void QBluetoothServiceDiscoveryAgentPrivate::_q_createdDevice(QDBusPendingCallWa
         qCDebug(QT_BT_BLUEZ) << Q_FUNC_INFO << "Discover restrictions:" << pattern;
 
         QDBusPendingReply<ServiceMap> discoverReply = device->DiscoverServices(pattern);
-        watcher = new QDBusPendingCallWatcher(discoverReply, q);
+        QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(discoverReply, q);
         QObject::connect(watcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
                          q, SLOT(_q_discoveredServices(QDBusPendingCallWatcher*)));
     }
@@ -488,8 +539,10 @@ void QBluetoothServiceDiscoveryAgentPrivate::_q_createdDevice(QDBusPendingCallWa
 // Bluez 4
 void QBluetoothServiceDiscoveryAgentPrivate::_q_discoveredServices(QDBusPendingCallWatcher *watcher)
 {
-    if (!device)
+    if (!device) {
+        watcher->deleteLater();
         return;
+    }
 
     qCDebug(QT_BT_BLUEZ) << Q_FUNC_INFO;
     Q_Q(QBluetoothServiceDiscoveryAgent);
@@ -609,14 +662,18 @@ void QBluetoothServiceDiscoveryAgentPrivate::performMinimalServiceDiscovery(cons
     }
 
     QStringList uuidStrings;
-    foreach (const QDBusObjectPath &path, reply.value().keys()) {
-        const InterfaceList ifaceList = reply.value().value(path);
-        foreach (const QString &iface, ifaceList.keys()) {
+
+    ManagedObjectList managedObjectList = reply.value();
+    for (ManagedObjectList::const_iterator it = managedObjectList.constBegin(); it != managedObjectList.constEnd(); ++it) {
+        const InterfaceList &ifaceList = it.value();
+
+        for (InterfaceList::const_iterator jt = ifaceList.constBegin(); jt != ifaceList.constEnd(); ++jt) {
+            const QString &iface = jt.key();
+            const QVariantMap &ifaceValues = jt.value();
+
             if (iface == QStringLiteral("org.bluez.Device1")) {
-                const QVariantMap details = ifaceList.value(iface);
-                if (deviceAddress.toString()
-                        == details.value(QStringLiteral("Address")).toString()) {
-                    uuidStrings = details.value(QStringLiteral("UUIDs")).toStringList();
+                if (deviceAddress.toString() == ifaceValues.value(QStringLiteral("Address")).toString()) {
+                    uuidStrings = ifaceValues.value(QStringLiteral("UUIDs")).toStringList();
                     break;
                 }
             }
