@@ -62,7 +62,8 @@ public:
         ServiceDiscovery,
     };
 
-    QBluetoothServiceDiscoveryAgentPrivate(const QBluetoothAddress &localAddress);
+    QBluetoothServiceDiscoveryAgentPrivate(QBluetoothServiceDiscoveryAgent *qp,
+                                           const QBluetoothAddress &localAddress);
 
     void startDeviceDiscovery();
     void stopDeviceDiscovery();
@@ -110,8 +111,9 @@ private:
     OSXBluetooth::ObjCScopedPointer<ObjCServiceInquiry> serviceInquiry;
 };
 
-QBluetoothServiceDiscoveryAgentPrivate::QBluetoothServiceDiscoveryAgentPrivate(const QBluetoothAddress &localAddress) :
-    q_ptr(0),
+QBluetoothServiceDiscoveryAgentPrivate::QBluetoothServiceDiscoveryAgentPrivate(
+    QBluetoothServiceDiscoveryAgent *qp, const QBluetoothAddress &localAddress) :
+    q_ptr(qp),
     error(QBluetoothServiceDiscoveryAgent::NoError),
     singleDevice(false),
     localAdapterAddress(localAddress),
@@ -423,15 +425,15 @@ void QBluetoothServiceDiscoveryAgentPrivate::serviceDiscoveryFinished()
 }
 
 QBluetoothServiceDiscoveryAgent::QBluetoothServiceDiscoveryAgent(QObject *parent)
-: QObject(parent), d_ptr(new QBluetoothServiceDiscoveryAgentPrivate(QBluetoothAddress()))
+: QObject(parent),
+  d_ptr(new QBluetoothServiceDiscoveryAgentPrivate(this, QBluetoothAddress()))
 {
-    d_ptr->q_ptr = this;
 }
 
 QBluetoothServiceDiscoveryAgent::QBluetoothServiceDiscoveryAgent(const QBluetoothAddress &deviceAdapter, QObject *parent)
-: QObject(parent), d_ptr(new QBluetoothServiceDiscoveryAgentPrivate(deviceAdapter))
+: QObject(parent),
+  d_ptr(new QBluetoothServiceDiscoveryAgentPrivate(this, deviceAdapter))
 {
-    d_ptr->q_ptr = this;
     if (!deviceAdapter.isNull()) {
         const QList<QBluetoothHostInfo> localDevices = QBluetoothLocalDevice::allDevices();
         foreach (const QBluetoothHostInfo &hostInfo, localDevices) {
@@ -495,9 +497,6 @@ QList<QBluetoothUuid> QBluetoothServiceDiscoveryAgent::uuidFilter() const
     address can only be set while there is no service discovery in progress; otherwise
     this function returns false.
 
-    On some platforms such as Blackberry the service discovery might lead to pairing requests.
-    Therefore it is not recommended to do service discoveries on all devices.
-
     \sa remoteAddress()
 */
 bool QBluetoothServiceDiscoveryAgent::setRemoteAddress(const QBluetoothAddress &address)
