@@ -46,7 +46,6 @@
 QT_BEGIN_NAMESPACE
 
 Q_DECLARE_LOGGING_CATEGORY(QT_BT)
-Q_DECLARE_LOGGING_CATEGORY(QT_BT_QNX)
 
 /*!
     \class QBluetoothSocket
@@ -60,7 +59,7 @@ Q_DECLARE_LOGGING_CATEGORY(QT_BT_QNX)
     \l {QBluetoothServiceInfo::RfcommProtocol}{RFCOMM}.
 
     \l {QBluetoothServiceInfo::L2capProtocol}{L2CAP} is a low level datagram-oriented Bluetooth socket.
-    Android and BlackBerry do not support \l {QBluetoothServiceInfo::L2capProtocol}{L2CAP} for socket
+    Android does not support \l {QBluetoothServiceInfo::L2capProtocol}{L2CAP} for socket
     connections.
 
     \l {QBluetoothServiceInfo::RfcommProtocol}{RFCOMM} is a reliable, stream-oriented socket. RFCOMM
@@ -73,6 +72,9 @@ Q_DECLARE_LOGGING_CATEGORY(QT_BT_QNX)
     If the \l {QBluetoothServiceInfo::Protocol}{Protocol} is not supported on a platform, calling
     \l connectToService() will emit a \l {QBluetoothSocket::UnsupportedProtocolError}{UnsupportedProtocolError} error.
 
+    \note QBluetoothSocket does not support synchronous read and write operations. Functions such
+    as \l waitForReadyRead() and \l waitForBytesWritten() are not implemented. I/O operations should be
+    performed using \l readyRead(), \l read() and \l write().
 */
 
 /*!
@@ -199,7 +201,7 @@ Q_DECLARE_LOGGING_CATEGORY(QT_BT_QNX)
     Although some platforms may differ the socket must generally be connected to guarantee
     the return of a valid port number.
 
-    On BlackBerry, Android and OS X, this feature is not supported and returns 0.
+    On Android and \macos, this feature is not supported and returns 0.
 */
 
 /*!
@@ -218,7 +220,7 @@ Q_DECLARE_LOGGING_CATEGORY(QT_BT_QNX)
     \fn quint16 QBluetoothSocket::peerPort() const
 
     Return the port number of the peer socket if available, otherwise returns 0.
-    On BlackBerry and Android, this feature is not supported.
+    On Android, this feature is not supported.
 */
 
 /*!
@@ -322,7 +324,7 @@ void QBluetoothSocket::connectToService(const QBluetoothServiceInfo &service, Op
         setSocketError(QBluetoothSocket::OperationError);
         return;
     }
-#if defined(QT_QNX_BLUETOOTH) || defined(QT_ANDROID_BLUETOOTH)
+#if defined(QT_ANDROID_BLUETOOTH)
     if (!d->ensureNativeSocket(service.socketProtocol())) {
         d->errorString = tr("Socket type not supported");
         setSocketError(QBluetoothSocket::UnsupportedProtocolError);
@@ -330,6 +332,15 @@ void QBluetoothSocket::connectToService(const QBluetoothServiceInfo &service, Op
     }
     d->connectToService(service.device().address(), service.serviceUuid(), openMode);
 #else
+    // Report this problem early:
+    if (socketType() == QBluetoothServiceInfo::UnknownProtocol) {
+        qCWarning(QT_BT) << "QBluetoothSocket::connectToService cannot "
+                            "connect with 'UnknownProtocol' type";
+        d->errorString = tr("Socket type not supported");
+        setSocketError(QBluetoothSocket::UnsupportedProtocolError);
+        return;
+    }
+
     if (service.protocolServiceMultiplexer() > 0) {
         if (!d->ensureNativeSocket(QBluetoothServiceInfo::L2capProtocol)) {
             d->errorString = tr("Unknown socket error");
@@ -346,7 +357,8 @@ void QBluetoothSocket::connectToService(const QBluetoothServiceInfo &service, Op
         d->connectToService(service.device().address(), service.serverChannel(), openMode);
     } else {
         // try doing service discovery to see if we can find the socket
-        if(service.serviceUuid().isNull()){
+        if (service.serviceUuid().isNull()
+                && !service.serviceClassUuids().contains(QBluetoothUuid::SerialPort)) {
             qCWarning(QT_BT) << "No port, no PSM, and no UUID provided, unable to connect";
             return;
         }
@@ -367,8 +379,8 @@ void QBluetoothSocket::connectToService(const QBluetoothServiceInfo &service, Op
     ConnectingState, and attempts to connect to \a address. If a connection is established,
     QBluetoothSocket enters Connected State and emits connected().
 
-    On BlackBerry and Android, the service connection can directly be established
-    using the UUID of the remote service. Therefore these platforms do not require
+    On Android, the service connection can directly be established
+    using the UUID of the remote service. Therefore the platforms does not require
     the \l ServiceLookupState and \l socketType() is always set to
     \l QBluetoothServiceInfo::RfcommProtocol.
 
@@ -390,7 +402,7 @@ void QBluetoothSocket::connectToService(const QBluetoothAddress &address, const
         return;
     }
 
-#if defined(QT_QNX_BLUETOOTH) || defined(QT_ANDROID_BLUETOOTH)
+#if defined(QT_ANDROID_BLUETOOTH)
     if (!d->ensureNativeSocket(QBluetoothServiceInfo::RfcommProtocol)) {
         d->errorString = tr("Socket type not supported");
         setSocketError(QBluetoothSocket::UnsupportedProtocolError);
@@ -398,6 +410,15 @@ void QBluetoothSocket::connectToService(const QBluetoothAddress &address, const
     }
     d->connectToService(address, uuid, openMode);
 #else
+    // Report this problem early, prevent device discovery:
+    if (socketType() == QBluetoothServiceInfo::UnknownProtocol) {
+        qCWarning(QT_BT) << "QBluetoothSocket::connectToService cannot "
+                            "connect with 'UnknownProtocol' type";
+        d->errorString = tr("Socket type not supported");
+        setSocketError(QBluetoothSocket::UnsupportedProtocolError);
+        return;
+    }
+
     QBluetoothServiceInfo service;
     QBluetoothDeviceInfo device(address, QString(), QBluetoothDeviceInfo::MiscellaneousDevice);
     service.setDevice(device);
@@ -416,7 +437,7 @@ void QBluetoothSocket::connectToService(const QBluetoothAddress &address, const
 
     At any point, the socket can emit error() to signal that an error occurred.
 
-    On BlackBerry and Android, a connection to a service can not be established using a port. Calling this function
+    On Android, a connection to a service can not be established using a port. Calling this function
     will emit a \l {QBluetoothSocket::ServiceNotFoundError}{ServiceNotFoundError}
 
     Note that most platforms require a pairing prior to connecting to the remote device. Otherwise
@@ -427,7 +448,7 @@ void QBluetoothSocket::connectToService(const QBluetoothAddress &address, const
 void QBluetoothSocket::connectToService(const QBluetoothAddress &address, quint16 port, OpenMode openMode)
 {
     Q_D(QBluetoothSocket);
-#if defined(QT_QNX_BLUETOOTH) || defined(QT_ANDROID_BLUETOOTH)
+#if defined(QT_ANDROID_BLUETOOTH)
     Q_UNUSED(port);
     Q_UNUSED(openMode);
     Q_UNUSED(address);
@@ -435,6 +456,15 @@ void QBluetoothSocket::connectToService(const QBluetoothAddress &address, quint1
     setSocketError(QBluetoothSocket::ServiceNotFoundError);
     qCWarning(QT_BT) << "Connecting to port is not supported";
 #else
+    // Report this problem early:
+    if (socketType() == QBluetoothServiceInfo::UnknownProtocol) {
+        qCWarning(QT_BT) << "QBluetoothSocket::connectToService cannot "
+                            "connect with 'UnknownProtocol' type";
+        d->errorString = tr("Socket type not supported");
+        setSocketError(QBluetoothSocket::UnsupportedProtocolError);
+        return;
+    }
+
     if (state() != QBluetoothSocket::UnconnectedState) {
         qCWarning(QT_BT)  << "QBluetoothSocket::connectToService called on busy socket";
         d->errorString = QBluetoothSocket::tr("Trying to connect while connection is in progress");
@@ -451,7 +481,7 @@ void QBluetoothSocket::connectToService(const QBluetoothAddress &address, quint1
     Returns the socket type. The socket automatically adjusts to the protocol
     offered by the remote service.
 
-    Blackberry and Android only support \l{QBluetoothServiceInfo::RfcommProtocol}{RFCOMM}
+    Android only support \l{QBluetoothServiceInfo::RfcommProtocol}{RFCOMM}
     based sockets.
 */
 QBluetoothServiceInfo::Protocol QBluetoothSocket::socketType() const
@@ -487,6 +517,60 @@ QString QBluetoothSocket::errorString() const
     return d->errorString;
 }
 
+/*!
+    Sets the preferred security parameter for the connection attempt to
+    \a flags. This value is incorporated when calling \l connectToService().
+    Therefore it is required to reconnect to change this parameter for an
+    existing connection.
+
+    On Bluez this property is set to QBluetooth::Authorization by default.
+
+    On \macos, this value is ignored as the platform does not permit access
+    to the security parameter of the socket. By default the platform prefers
+    secure/encrypted connections though and therefore this function always
+    returns \l QBluetooth::Secure.
+
+    Android only supports two levels of security (secure and non-secure). If this flag is set to
+    \l QBluetooth::NoSecurity the socket object will not employ any authentication or encryption.
+    Any other security flag combination will trigger a secure Bluetooth connection.
+    This flag is set to \l QBluetooth::Secure by default.
+
+    \note A secure connection requires a pairing between the two devices. On
+    some platforms, the pairing is automatically initiated during the establishment
+    of the connection. Other platforms require the application to manually trigger
+    the pairing before attempting to connect.
+
+    \sa preferredSecurityFlags()
+
+    \since 5.6
+*/
+void QBluetoothSocket::setPreferredSecurityFlags(QBluetooth::SecurityFlags flags)
+{
+    Q_D(QBluetoothSocket);
+    if (d->secFlags != flags)
+        d->secFlags = flags;
+}
+
+/*!
+    Returns the security parameters used for the initial connection
+    attempt.
+
+    The security parameters may be renegotiated between the two parties
+    during or after the connection has been established. If such a change happens
+    it is not reflected in the value of this flag.
+
+    On \macos, this flag is always set to \l QBluetooth::Secure.
+
+    \sa setPreferredSecurityFlags()
+
+    \since 5.6
+*/
+QBluetooth::SecurityFlags QBluetoothSocket::preferredSecurityFlags() const
+{
+    Q_D(const QBluetoothSocket);
+    return d->secFlags;
+}
+
 /*!
     Sets the socket state to \a state.
 */
@@ -537,7 +621,7 @@ void QBluetoothSocket::doDeviceDiscovery(const QBluetoothServiceInfo &service, O
     Q_D(QBluetoothSocket);
 
     setSocketState(QBluetoothSocket::ServiceLookupState);
-    qCDebug(QT_BT) << "Starting discovery";
+    qCDebug(QT_BT) << "Starting Bluetooth Socket discovery";
 
     if(d->discoveryAgent) {
         d->discoveryAgent->stop();
@@ -554,11 +638,12 @@ void QBluetoothSocket::doDeviceDiscovery(const QBluetoothServiceInfo &service, O
 
     d->openMode = openMode;
 
+    QList<QBluetoothUuid> filterUuids = service.serviceClassUuids();
     if(!service.serviceUuid().isNull())
-        d->discoveryAgent->setUuidFilter(service.serviceUuid());
+        filterUuids.append(service.serviceUuid());
 
-    if(!service.serviceClassUuids().isEmpty())
-        d->discoveryAgent->setUuidFilter(service.serviceClassUuids());
+    if (!filterUuids.isEmpty())
+        d->discoveryAgent->setUuidFilter(filterUuids);
 
     // we have to ID the service somehow
     Q_ASSERT(!d->discoveryAgent->uuidFilter().isEmpty());
