@@ -194,18 +194,17 @@ QT_BEGIN_NAMESPACE
     \sa discoverServices(), error()
 */
 
-namespace {
-class QLowEnergyControllerMetaTypes
+void registerQLowEnergyControllerMetaType()
 {
-public:
-    QLowEnergyControllerMetaTypes()
-    {
+    static bool initDone = false;
+    if (!initDone) {
         qRegisterMetaType<QLowEnergyController::ControllerState>();
         qRegisterMetaType<QLowEnergyController::Error>();
+        initDone = true;
     }
-} qLowEnergyControllerMetaTypes;
 }
 
+
 void QLowEnergyControllerPrivate::setError(
         QLowEnergyController::Error newError)
 {
@@ -222,8 +221,13 @@ void QLowEnergyControllerPrivate::setError(
     case QLowEnergyController::NetworkError:
         errorString = QLowEnergyController::tr("Error occurred during connection I/O");
         break;
-    case QLowEnergyController::UnknownError:
+    case QLowEnergyController::ConnectionError:
+        errorString = QLowEnergyController::tr("Error occurred trying to connect to remote device.");
+        break;
+    case QLowEnergyController::NoError:
+        return;
     default:
+    case QLowEnergyController::UnknownError:
         errorString = QLowEnergyController::tr("Unknown Error");
         break;
     }
@@ -339,13 +343,18 @@ quint16 QLowEnergyControllerPrivate::updateValueOfCharacteristic(
         QLowEnergyHandle charHandle,const QByteArray &value, bool appendValue)
 {
     QSharedPointer<QLowEnergyServicePrivate> service = serviceForHandle(charHandle);
-    if (!service.isNull() && service->characteristicList.contains(charHandle)) {
-        if (appendValue)
-            service->characteristicList[charHandle].value += value;
-        else
-            service->characteristicList[charHandle].value = value;
+    if (!service.isNull()) {
+        CharacteristicDataMap::iterator charIt = service->characteristicList.find(charHandle);
+        if (charIt != service->characteristicList.end()) {
+            QLowEnergyServicePrivate::CharData &charDetails = charIt.value();
 
-        return service->characteristicList[charHandle].value.size();
+            if (appendValue)
+                charDetails.value += value;
+            else
+                charDetails.value = value;
+
+            return charDetails.value.size();
+        }
     }
 
     return 0;
@@ -359,18 +368,26 @@ quint16 QLowEnergyControllerPrivate::updateValueOfDescriptor(
         const QByteArray &value, bool appendValue)
 {
     QSharedPointer<QLowEnergyServicePrivate> service = serviceForHandle(charHandle);
-    if (service.isNull() || !service->characteristicList.contains(charHandle))
-        return 0;
+    if (!service.isNull()) {
+        CharacteristicDataMap::iterator charIt = service->characteristicList.find(charHandle);
+        if (charIt != service->characteristicList.end()) {
+            QLowEnergyServicePrivate::CharData &charDetails = charIt.value();
 
-    if (!service->characteristicList[charHandle].descriptorList.contains(descriptorHandle))
-        return 0;
+            DescriptorDataMap::iterator descIt = charDetails.descriptorList.find(descriptorHandle);
+            if (descIt != charDetails.descriptorList.end()) {
+                QLowEnergyServicePrivate::DescData &descDetails = descIt.value();
 
-    if (appendValue)
-        service->characteristicList[charHandle].descriptorList[descriptorHandle].value += value;
-    else
-        service->characteristicList[charHandle].descriptorList[descriptorHandle].value = value;
+                if (appendValue)
+                    descDetails.value += value;
+                else
+                    descDetails.value = value;
 
-    return service->characteristicList[charHandle].descriptorList[descriptorHandle].value.size();
+                return descDetails.value.size();
+            }
+        }
+    }
+
+    return 0;
 }
 
 /*!
@@ -395,6 +412,7 @@ QLowEnergyController::QLowEnergyController(
     d->remoteDevice = remoteDevice;
     d->localAdapter = QBluetoothLocalDevice().address();
     d->addressType = QLowEnergyController::PublicAddress;
+    d->init();
 }
 
 /*!
@@ -420,6 +438,7 @@ QLowEnergyController::QLowEnergyController(
     d->localAdapter = QBluetoothLocalDevice().address();
     d->addressType = QLowEnergyController::PublicAddress;
     d->remoteName = remoteDeviceInfo.name();
+    d->init();
 }
 
 /*!
@@ -447,6 +466,7 @@ QLowEnergyController::QLowEnergyController(
     d->q_ptr = this;
     d->remoteDevice = remoteDevice;
     d->localAdapter = localDevice;
+    d->init();
 }
 
 /*!
@@ -641,11 +661,15 @@ QLowEnergyService *QLowEnergyController::createServiceObject(
         const QBluetoothUuid &serviceUuid, QObject *parent)
 {
     Q_D(QLowEnergyController);
-    if (!d->serviceList.contains(serviceUuid))
-        return 0;
 
-    QLowEnergyService *service = new QLowEnergyService(
-                d->serviceList.value(serviceUuid), parent);
+    QLowEnergyService *service = Q_NULLPTR;
+
+    ServiceDataMap::const_iterator it = d->serviceList.constFind(serviceUuid);
+    if (it != d->serviceList.constEnd()) {
+        const QSharedPointer<QLowEnergyServicePrivate> &serviceData = it.value();
+
+        service = new QLowEnergyService(serviceData, parent);
+    }
 
     return service;
 }
